@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -62,10 +62,7 @@ product endorsement purposes.
 #include "hdf5.h"
 
 /* useful macro for comparing HDF5 versions */
-#define HDF5_VERSION_GE(Maj,Min,Rel)  \
-        (((H5_VERS_MAJOR==Maj) && (H5_VERS_MINOR==Min) && (H5_VERS_RELEASE>=Rel)) || \
-         ((H5_VERS_MAJOR==Maj) && (H5_VERS_MINOR>Min)) || \
-         (H5_VERS_MAJOR>Maj))
+#include "hdf5_version_ge.h"
 
 #if HDF5_VERSION_GE(1,8,4)
 
@@ -488,7 +485,11 @@ static int H5FD_silo_cmp(const H5FD_t *_f1, const H5FD_t *_f2);
 static herr_t H5FD_silo_query(const H5FD_t *_f1, unsigned long *flags);
 static haddr_t H5FD_silo_get_eoa(const H5FD_t *_file, H5FD_mem_t type);
 static herr_t H5FD_silo_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t addr);
+#if HDF5_VERSION_GE(1,10,4)
+static haddr_t H5FD_silo_get_eof(const H5FD_t *_file, H5FD_mem_t type);
+#else
 static haddr_t H5FD_silo_get_eof(const H5FD_t *_file);
+#endif
 static herr_t  H5FD_silo_get_handle(H5FD_t *_file, hid_t fapl, void** file_handle);
 static herr_t H5FD_silo_read(H5FD_t *lf, H5FD_mem_t type, hid_t fapl_id, haddr_t addr,
                 size_t size, void *buf);
@@ -496,10 +497,56 @@ static herr_t H5FD_silo_write(H5FD_t *lf, H5FD_mem_t type, hid_t fapl_id, haddr_
                 size_t size, const void *buf);
 static herr_t H5FD_silo_truncate(H5FD_t *_file, hid_t dxpl_id, hbool_t closing);
 
+#warning REMOVE ME
+#if 0
+typedef struct H5FD_class_t {
+    const char *name;
+    haddr_t maxaddr;
+    H5F_close_degree_t fc_degree;
+    herr_t  (*terminate)(void);
+    hsize_t (*sb_size)(H5FD_t *file);
+    herr_t  (*sb_encode)(H5FD_t *file, char *name/*out*/,
+                         unsigned char *p/*out*/);
+    herr_t  (*sb_decode)(H5FD_t *f, const char *name, const unsigned char *p);
+    size_t  fapl_size;
+    void *  (*fapl_get)(H5FD_t *file);
+    void *  (*fapl_copy)(const void *fapl);
+    herr_t  (*fapl_free)(void *fapl);
+    size_t  dxpl_size;
+    void *  (*dxpl_copy)(const void *dxpl);
+    herr_t  (*dxpl_free)(void *dxpl);
+    H5FD_t *(*open)(const char *name, unsigned flags, hid_t fapl,
+                    haddr_t maxaddr);
+    herr_t  (*close)(H5FD_t *file);
+    int     (*cmp)(const H5FD_t *f1, const H5FD_t *f2);
+    herr_t  (*query)(const H5FD_t *f1, unsigned long *flags);
+    herr_t  (*get_type_map)(const H5FD_t *file, H5FD_mem_t *type_map);
+    haddr_t (*alloc)(H5FD_t *file, H5FD_mem_t type, hid_t dxpl_id, hsize_t size);
+    herr_t  (*free)(H5FD_t *file, H5FD_mem_t type, hid_t dxpl_id,
+                    haddr_t addr, hsize_t size);
+    haddr_t (*get_eoa)(const H5FD_t *file, H5FD_mem_t type);
+    herr_t  (*set_eoa)(H5FD_t *file, H5FD_mem_t type, haddr_t addr);
+    haddr_t (*get_eof)(const H5FD_t *file, H5FD_mem_t type);
+    herr_t  (*get_handle)(H5FD_t *file, hid_t fapl, void**file_handle);
+    herr_t  (*read)(H5FD_t *file, H5FD_mem_t type, hid_t dxpl,
+                    haddr_t addr, size_t size, void *buffer);
+    herr_t  (*write)(H5FD_t *file, H5FD_mem_t type, hid_t dxpl,
+                     haddr_t addr, size_t size, const void *buffer);
+    herr_t  (*flush)(H5FD_t *file, hid_t dxpl_id, hbool_t closing);
+    herr_t  (*truncate)(H5FD_t *file, hid_t dxpl_id, hbool_t closing);
+    herr_t  (*lock)(H5FD_t *file, hbool_t rw);
+    herr_t  (*unlock)(H5FD_t *file);
+    H5FD_mem_t fl_map[H5FD_MEM_NTYPES];
+} H5FD_class_t;
+#endif
+
 static const H5FD_class_t H5FD_silo_g = {
     "silo",				        /*name			*/
     MAXADDR,				        /*maxaddr		*/
     H5F_CLOSE_WEAK,				/* fc_degree		*/
+#if HDF5_VERSION_GE(1,10,4)
+    0,                                          /* terminate            */
+#endif
     H5FD_silo_sb_size,                          /*sb_size               */
     H5FD_silo_sb_encode,                        /*sb_encode             */
     H5FD_silo_sb_decode,                        /*sb_decode             */
@@ -573,7 +620,7 @@ static void update_hotblock_stats(H5FD_silo_t *file, hsize_t id, int dir, float
         if (file->stats.num_hot_blocks == file->stats.max_hot_blocks)
         {
             int new_max = file->stats.max_hot_blocks * 2 + 1;
-            hbl = realloc(file->stats.hot_block_list, sizeof(silo_vfd_hot_block_stats_t)*new_max);
+            hbl = (silo_vfd_hot_block_stats_t *)realloc(file->stats.hot_block_list, sizeof(silo_vfd_hot_block_stats_t)*new_max);
             file->stats.max_hot_blocks = new_max;
             file->stats.hot_block_list = hbl;
         }
@@ -1925,8 +1972,13 @@ H5FD_silo_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t addr)
  *-------------------------------------------------------------------------
  */
 static haddr_t
+#if HDF5_VERSION_GE(1,10,4)
+H5FD_silo_get_eof(const H5FD_t *_file, H5FD_mem_t type)
+#else
 H5FD_silo_get_eof(const H5FD_t *_file)
+#endif
 {
+#warning TAKE ADVANTAGE OF TYPE INFO HERE
     const H5FD_silo_t	*file = (const H5FD_silo_t *)_file;
 
     return(MAX(file->eof, file->eoa));
