@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -561,7 +561,7 @@ silo_GetHypercube(void *sink, void *source, int *dims, int ndims, int *start,
    char          *local_sink, *local_source; /* Index by bytes */
 
    local_sink = (char*)sink;
-   local_source = source;
+   local_source = (char *)source;
 
    /* If sink is 1D, just do a memory copy from source */
    if (ndims == 1) {
