@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -492,6 +492,10 @@ _lite_PD_rd_symt (PDBfile *file) {
  * Modifications:
  *      Sean Ahern, Wed Apr 12 10:49:15 PDT 2000
  *      Removed some compiler warnings.
+ *
+ *      Mark C. Miller, Tue Sep 15 18:35:42 PDT 2020
+ *      Create temp. copy of _PD_cast_lst to avoid memory corruption
+ *      when its used in both file and host charts.
  *-------------------------------------------------------------------------*/
 int
 _lite_PD_rd_chrt (PDBfile *file) {
@@ -500,6 +504,7 @@ _lite_PD_rd_chrt (PDBfile *file) {
    FILE *fp;
    memdes *desc, *lst, *prev;
    long i, chrt_sz;
+   char **_PD_cast_lst_cpy;
 
    fp = file->stream;
 
@@ -540,15 +545,25 @@ _lite_PD_rd_chrt (PDBfile *file) {
    if (_PD_has_dirs) lite_PD_defncv(file, "Directory", 1, 0);
    _PD_has_dirs = FALSE;
 
+   /*
+    * Copy the cast list.
+    */
+   _PD_cast_lst_cpy = FMAKE_N(char *, _PD_n_casts, "_PD_RD_CHART:cast-list-copy");
+   for (i = 0L; i < _PD_n_casts; i += 3) {
+      _PD_cast_lst_cpy[i  ] = lite_SC_strsavef(_PD_cast_lst[i  ], "char*:_PD_RD_CHART:cast-list-copy-local1");
+      _PD_cast_lst_cpy[i+1] = lite_SC_strsavef(_PD_cast_lst[i+1], "char*:_PD_RD_CHART:cast-list-copy-local2");
+      _PD_cast_lst_cpy[i+2] = lite_SC_strsavef(_PD_cast_lst[i+2], "char*:_PD_RD_CHART:cast-list-copy-local3");
+   }
+
    /*
     * Check the casts for the file->chart.
     */
    _lite_PD_check_casts(file->chart, _PD_cast_lst, _PD_n_casts);
 
    /*
-    * Check the casts for the file->host_chart.
+    * Check the casts for the file->host_chart using the copied cast list.
     */
-   _lite_PD_check_casts(file->host_chart, _PD_cast_lst, _PD_n_casts);
+   _lite_PD_check_casts(file->host_chart, _PD_cast_lst_cpy, _PD_n_casts);
 
    /*
     * Clean up the mess.
@@ -556,8 +571,11 @@ _lite_PD_rd_chrt (PDBfile *file) {
    for (i = 0L; i < _PD_n_casts; i += 3) {
       SFREE(_PD_cast_lst[i]);
       SFREE(_PD_cast_lst[i+1]);
+      SFREE(_PD_cast_lst_cpy[i]);
+      SFREE(_PD_cast_lst_cpy[i+1]);
    }
    SFREE(_PD_cast_lst);
+   SFREE(_PD_cast_lst_cpy);
    _PD_n_casts = 0L;
 
    SFREE(_lite_PD_tbuffer);
@@ -1344,6 +1362,9 @@ _lite_PD_init_chrt (PDBfile *file) {
  *   Mark C. Miller, Tue Nov 17 22:23:42 PST 2009
  *   Changed support for long long to match more closely what PDB
  *   proper does.
+ *
+ *   Mark C. Miller, Fri May 11 15:59:35 PDT 2018
+ *   Added "char *" entry per Dale Slone.
  *-------------------------------------------------------------------------
  */
 void
@@ -1369,6 +1390,14 @@ _lite_PD_setup_chart (HASHTAB *chart, data_standard *fstd, data_standard *hstd,
    _lite_PD_defstr(chart, "char", falign->char_alignment,
                    1L, -1, conv, NULL, NULL);
 
+  if (flag) {
+      PD_COMPARE_CHAR_STD(conv, fstd, hstd, falign, halign);
+   } else {
+      conv = FALSE;
+   }
+   _lite_PD_defstr(chart, "char *", falign->char_alignment,
+                   1L, -1, conv, NULL, NULL);
+
    if (flag) {
       PD_COMPARE_SHORT_STD(conv, fstd, hstd, falign, halign);
    } else {
@@ -1757,13 +1786,13 @@ void
 _lite_PD_eod (PDBfile *file) {
 
    FILE *fp;
-   long old, new;
+   long old_one, new_one;
 
    fp  = file->stream;
-   old = file->chrtaddr;
-   new = io_tell(fp);
+   old_one = file->chrtaddr;
+   new_one = io_tell(fp);
 
-   file->chrtaddr = MAX(old, new);
+   file->chrtaddr = MAX(old_one, new_one);
 }
 #endif /* PDB_WRITE */
 
@@ -1904,8 +1933,8 @@ _lite_PD_wr_symt (PDBfile *file) {
          for (lst = PD_entry_dimensions(ep); lst != NULL; lst = lst->next) {
             if ((flag == ROW_MAJOR_ORDER) ||
                 ((flag == COLUMN_MAJOR_ORDER) && (lst->next == NULL))) {
-               stride = nt/(lst->number);
-               ne     = nb/stride;
+               stride = lst->number?nt/(lst->number):0;
+               ne     = stride?nb/stride:0;
                flag   = FALSE;
             } else {
                ne = lst->number;
