@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -297,7 +297,7 @@ _lite_PD_hyper_read (PDBfile *file, char *name, char *outtype,
     */
    tep = _lite_PD_mk_syment(NULL, 0L, 0L, &(ep->indirects), NULL);
 
-   nrd = _PD_rd_hyper_index(file, tep, vr, pi,
+   nrd = _PD_rd_hyper_index(file, tep, (char *)vr, pi,
                             PD_entry_type(ep), outtype,
                             PD_entry_address(ep),
                             PD_entry_blocks(ep), hbyt, fbyt);
@@ -1340,6 +1340,21 @@ _PD_compute_hyper_strides (PDBfile *file, char *ind, dimdes *dims, int *pnd) {
  *-------------------------------------------------------------------------
  */
 #ifdef PDB_WRITE
+static int is_empty_case(lite_SC_byte const *vr, dimdes const *dims, long num)
+{
+    if (vr == 0 && dims == 0 && num == 1) return 1;
+
+    if (vr != 0) return 0;
+    if (num > 0) return 0;
+    if (!dims) {printf("empty case for !dims\n"); return 1;}
+    while (dims)
+    {
+        if (dims->number>0) return 0;
+        dims = dims->next;
+    }
+    return 1;
+}
+
 int
 _lite_PD_hyper_write (PDBfile *file, char *name, syment *ep, lite_SC_byte *vr,
                       char *intype) {
@@ -1356,8 +1371,18 @@ _lite_PD_hyper_write (PDBfile *file, char *name, syment *ep, lite_SC_byte *vr,
    slen = strlen(s);
    c = slen > 0 ? s[slen-1] : 0;
    if (((c != ')') && (c != ']')) || (dims == NULL)) {
-      return(_lite_PD_wr_syment(file, vr, PD_entry_number(ep),
+      if (is_empty_case(vr, dims, PD_entry_number(ep)))
+      {
+          long ret = _lite_PD_wr_syment(file, (char *)vr, 0,
+                                intype, PD_entry_type(ep));
+          PD_entry_number(ep) = 0;
+          return ret==0;
+      }
+      else
+      {
+          return(_lite_PD_wr_syment(file, (char *)vr, PD_entry_number(ep),
                                 intype, PD_entry_type(ep)));
+      }
    }
 
    if (_lite_PD_indirection(PD_entry_type(ep))) {
@@ -1395,7 +1420,7 @@ _lite_PD_hyper_write (PDBfile *file, char *name, syment *ep, lite_SC_byte *vr,
                     PD_WRITE);
    }
 
-   _PD_wr_hyper_index(file, vr, pi, intype,
+   _PD_wr_hyper_index(file, (char *)vr, pi, intype,
                       PD_entry_type(ep), PD_entry_address(ep),
                       PD_entry_blocks(ep), hbyt, fbyt);
 
@@ -1727,7 +1752,7 @@ _PD_wr_leaf_members (PDBfile *file, char *intype, char *outtype,
                        PD_WRITE);
       }
       vbuf     = buf;
-      svr      = vr;
+      svr      = (char *)vr;
       in_offs  = 0L;
       out_offs = 0L;
       lite_PD_convert(&vbuf, &svr, intype, outtype, nitems,
