@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -87,7 +87,7 @@ UpdateTree(DBexprnode *tree, const char t, int v, char *s)
         if (s)
             strncpy(newnode->sval, s, sizeof(newnode->sval)-1);
         else
-            strcpy(newnode->sval, "(null)");
+            newnode->sval[0] = '\0';
     }
 
     newnode->left = 0;
@@ -175,7 +175,6 @@ BuildExprTree(const char **porig)
                 char typec = *p;
                 char tokbuf[129];
                 char *tp = tokbuf;
-                DBexprnode *subtree;
                 p++;
                 while (*p != '[')
                     *tp++ = *p++;
@@ -183,14 +182,6 @@ BuildExprTree(const char **porig)
                 *tp = '\0';
                 errno = 0;
                 tree = UpdateTree(tree, typec, 0, tokbuf);
-                p++;
-                subtree = BuildExprTree(&p);
-                if (tree->left == 0)
-                    tree->left = subtree;
-                else if (tree->right == 0)
-                    tree->right = subtree;
-                else
-                    free(subtree); /* should never hit this case */
                 break;
             }
 
@@ -202,7 +193,7 @@ BuildExprTree(const char **porig)
                 break;
             }
 
-            case '(':
+            case '(': case '[':
             {
                 DBexprnode *subtree;
                 p++;
@@ -257,11 +248,15 @@ BuildExprTree(const char **porig)
 }
 
 /* very simple circular cache for a handful of embedded strings */
-static int SaveInternalString(DBnamescheme *ns, const char *sval)
+static int SaveInternalString(DBnamescheme const *ns, const char *sval)
 {
-    int modn = ns->nembed++ % DB_MAX_EXPSTRS;
-    FREE(ns->embedstrs[modn]);
-    ns->embedstrs[modn] = STRDUP(sval);
+    /* The modn/embedns portion of a namescheme is 'internal' state
+       allowed to disobey const rules */
+    DBnamescheme *non_const_ns = (DBnamescheme*) ns;
+    int modn = non_const_ns->nembed++ % DB_MAX_EXPNS;
+    if (non_const_ns->embedns[modn])
+        DBFreeNamescheme(non_const_ns->embedns[modn]);
+    non_const_ns->embedns[modn] = DBMakeNamescheme(sval);
     return modn;
 }
 
@@ -270,8 +265,10 @@ static int SaveInternalString(DBnamescheme *ns, const char *sval)
 static char * retstrbuf[DB_MAX_RETSTRS];
 static char * SaveReturnedString(char const * retstr)
 {
-    static unsigned int n = 0;
-    int modn = n++ % DB_MAX_RETSTRS;
+    static size_t n = 0;
+    size_t modn;
+
+    /* Hack to cleanup when really needed */
     if (retstr == 0)
     {
         for (n = 0; n < DB_MAX_RETSTRS; n++)
@@ -279,13 +276,16 @@ static char * SaveReturnedString(char const * retstr)
         n = 0;
         return 0;
     }
+
+    modn = n % DB_MAX_RETSTRS;
+    n++;
     FREE(retstrbuf[modn]);
     retstrbuf[modn] = STRDUP(retstr);
     return retstrbuf[modn];
 }
 
 static int
-EvalExprTree(DBnamescheme *ns, DBexprnode *tree, int n)
+EvalExprTree(DBnamescheme const *ns, DBexprnode *tree, int n)
 {
     if (tree == 0)
         return 0;
@@ -314,21 +314,28 @@ EvalExprTree(DBnamescheme *ns, DBexprnode *tree, int n)
     }
     else if (tree->left != 0 && tree->right != 0)
     {
-        int vc = 0, vl, vr;
+        int vc = 0, vl = 0, vr = 0;
         if (tree->type == '?')
         {
             vc = EvalExprTree(ns, tree->left, n);
             tree = tree->right;
+            if (vc) 
+                vl = EvalExprTree(ns, tree->left, n);
+            else
+                vr = EvalExprTree(ns, tree->right, n);
+        }
+        else
+        {
+            vl = EvalExprTree(ns, tree->left, n);
+            vr = EvalExprTree(ns, tree->right, n);
         }
-        vl = EvalExprTree(ns, tree->left, n);
-        vr = EvalExprTree(ns, tree->right, n);
         switch (tree->type)
         {
             case '+': return vl + vr;
             case '-': return vl - vr;
             case '*': return vl * vr;
-            case '/': return vl / vr;
-            case '%': return vl % vr;
+            case '/': return (vr != 0 ? vl / vr : 1);
+            case '%': return (vr != 0 ? vl % vr : 1);
             case '|': return vl | vr;
             case '&': return vl & vr;
             case '^': return vl ^ vr;
@@ -339,7 +346,7 @@ EvalExprTree(DBnamescheme *ns, DBexprnode *tree, int n)
 }
 
 PUBLIC DBnamescheme *
-DBMakeNamescheme(const char *fmt, ...)
+DBMakeNamescheme(char const *fmt, ...)
 {
     va_list ap;
     int i, j, k, n, pass, ncspecs, done, saved_narrefs;
@@ -353,10 +360,20 @@ DBMakeNamescheme(const char *fmt, ...)
 
     /* Start by allocating an empty name scheme */
     rv = DBAllocNamescheme();
-    
-    /* set the delimeter character */
-    rv->delim = fmt[0];
 
+    // set the delimeter character
+    n = 0;
+    while (fmt[n] != '\0')
+    {
+        if (fmt[n] == '%' && fmt[n+1] != '%')
+            break;
+        n++;
+    }
+    if (fmt[n] == '%') // have at least one conversion spec
+        rv->delim = fmt[0];
+    else
+        rv->delim = '\0';
+    
     /* compute length up to max of 4096 of initial segment of fmt representing
        the printf-style format string. */
     n = 1;
@@ -394,8 +411,14 @@ DBMakeNamescheme(const char *fmt, ...)
     rv->fmtptrs[rv->ncspecs] = &(rv->fmt[n+1]);
 
     /* If there are no conversion specs., we have nothing to do */
+    /* However, in this case, assume the first char is a real char. */
     if (rv->ncspecs == 0)
+    {
+        free(rv->fmt);
+        rv->fmt = STRNDUP(&fmt[0],n);
+        rv->fmtlen = n;
         return rv;
+    }
 
     /* Make a pass through rest of fmt string to count array refs in the
        expression substrings. */
@@ -467,21 +490,16 @@ DBMakeNamescheme(const char *fmt, ...)
                 if (strncmp(&fmt[i+1],rv->arrnames[k],j-1) == 0)
                     break;
             }
-            if (k == rv->narrefs)
+            if (k < rv->narrefs) /* this ext. array name has already been */
+            {                    /* seen and is being used multiple times. */
+                saved_narrefs--;
+            }
+            else
             {
                 rv->arrnames[k] = STRNDUP(&fmt[i+1], j-1);
                 if (!dbfile)
                 {
                     rv->arrvals[k] = va_arg(ap, void *);
-#if 0
-                    if (rv->arrvals[k] < (void*) 0x0000FFFF)
-                    {
-                        DBFreeNamescheme(rv);
-                        rv = 0;
-                        done = 1;
-                        continue;
-                    }
-#endif
                 }
                 else
                 {
@@ -495,7 +513,7 @@ DBMakeNamescheme(const char *fmt, ...)
                         {
                             char **tmp = NULL;
                             rv->arrsizes[k] = -1; /* initialize to 'unknown size' */
-                            tmp = DBStringListToStringArray((char*)rv->arrvals[k], &(rv->arrsizes[k]), 0, 0);
+                            tmp = DBStringListToStringArray((char*)rv->arrvals[k], &(rv->arrsizes[k]), 0);
                             FREE(rv->arrvals[k]);
                             rv->arrvals[k] = tmp;
                         }
@@ -516,8 +534,8 @@ DBMakeNamescheme(const char *fmt, ...)
         {
             rv->exprstrs[ncspecs] = STRNDUP(&fmt[n+1],i-(n+1));
             ncspecs++;
-            if (fmt[i] == '\0' ||
-                (fmt[i] == rv->delim && fmt[i] == '\0'))
+            if ((fmt[i] == '\0') ||
+                (fmt[i] == rv->delim && fmt[i+1] == '\0'))
                 done = 1;
             n = i;
         }
@@ -531,20 +549,32 @@ DBMakeNamescheme(const char *fmt, ...)
         rv = 0;
     }
 
+    for (i = 0; rv && i < rv->ncspecs; i++)
+    {
+        if (!rv->exprstrs[i])
+        {
+            DBFreeNamescheme(rv);
+            rv = 0;
+            break;
+        }
+    }
+
     return rv;
 }
 
 PUBLIC const char *
-DBGetName(DBnamescheme *ns, int natnum)
+DBGetName(DBnamescheme const *ns, int natnum)
 {
     char *currentExpr, *tmpExpr;
-    static char retval[1024];
+    char retval[1024];
     int i;
 
     /* a hackish way to cleanup the saved returned string buffer */
-    if (ns == 0 && natnum == 0) return SaveReturnedString(0);
+    if (ns == 0 && natnum == -1) return SaveReturnedString(0);
     if (ns == 0) return SaveReturnedString("");
 
+    if (!ns->fmt) return "";
+
     retval[0] = '\0';
     strncat(retval, ns->fmt, ns->fmtptrs[0] - ns->fmt);
     for (i = 0; i < ns->ncspecs; i++)
@@ -567,8 +597,8 @@ DBGetName(DBnamescheme *ns, int natnum)
         theVal = EvalExprTree(ns, exprtree, natnum);
         FreeTree(exprtree);
         strncpy(tmpfmt, ns->fmtptrs[i], ns->fmtptrs[i+1] - ns->fmtptrs[i]);
-        if (strncmp(tmpfmt, "%s", 2) == 0 && 0 <= theVal && theVal < DB_MAX_EXPSTRS)
-            sprintf(tmp, tmpfmt, ns->embedstrs[theVal]);
+        if (strncmp(tmpfmt, "%s", 2) == 0 && 0 <= theVal && theVal < DB_MAX_EXPNS)
+            sprintf(tmp, tmpfmt, DBGetName(ns->embedns[theVal],natnum));
         else
             sprintf(tmp, tmpfmt, theVal);
         strcat(retval, tmp);
@@ -576,3 +606,41 @@ DBGetName(DBnamescheme *ns, int natnum)
     }
     return SaveReturnedString(retval);
 }
+
+PUBLIC int
+DBGetIndex(DBnamescheme const *ns, int natnum)
+{
+    char const *name_str = DBGetName(ns, natnum);
+    int i = 0;
+
+    if (!name_str) return -1;
+
+    while (name_str[i] && !(strchr("0123456789+-",      name_str[i  ]) &&
+                            strchr("0123456789.aAbBcCdDeEfFxX+-", name_str[i+1])))
+        i++;
+
+    if (!name_str[i]) return -1;
+
+    return (int) strtol(&name_str[i], 0, 10);
+}
+
+PUBLIC char const *
+DBSPrintf(char const *fmt, ...)
+{
+    static char strbuf[2048];
+    static size_t const nmax = sizeof(strbuf);
+    va_list ap;
+    int n, en;
+
+    if (!fmt) return SaveReturnedString(0);
+
+    va_start(ap, fmt);
+    n = vsnprintf(strbuf, nmax, fmt, ap);
+    en = errno;
+    va_end(ap);
+
+    if (n < 0)
+        snprintf(strbuf, nmax, "DBsprintf_failed_with_error_%s", strerror(en));
+
+    return SaveReturnedString(strbuf);
+}
