@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -69,7 +69,7 @@ static char   *dir_names[] =
 {"almansi", "green", "inf_strain",
  "nodal", "shell", "stress", "rates"};
 
-PRIVATE int db_taur_cd(TAURUSfile *taurus, char *path);
+PRIVATE int db_taur_cd(TAURUSfile *taurus, char const *path);
 PRIVATE int db_taur_pwd(TAURUSfile *taurus, char *path);
 INTERNAL void db_taur_extface(int *znodelist, int nnodes,
                  int nzones, int *matlist, int **fnodelist,
@@ -118,7 +118,7 @@ db_taur_InitCallbacks (DBfile *dbfile)
     dbfile->pub.r_var = db_taur_ReadVar;
     dbfile->pub.newtoc = db_taur_NewToc;
     dbfile->pub.module = db_taur_Filters;
-    dbfile->pub.inqvartype = (DBObjectType(*)(struct DBfile *, char*)) db_taur_InqVartype;
+    dbfile->pub.inqvartype = db_taur_InqVartype;
 }
 
 /*-------------------------------------------------------------------------
@@ -231,7 +231,7 @@ get_next_int (void)
  *-------------------------------------------------------------------------*/
 /* ARGSUSED */
 PUBLIC DBfile *
-db_taur_Open(char *name, int mode, int subtype)
+db_taur_Open(char const *name, int mode, int subtype)
 {
     TAURUSfile    *taurus;
     DBfile_taur   *dbfile;
@@ -348,7 +348,7 @@ db_taur_GetDir(DBfile *_dbfile, char *path)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK int
-db_taur_SetDir(DBfile *_dbfile, char *path)
+db_taur_SetDir(DBfile *_dbfile, char const *path)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     char          *me = "db_taur_SetDir";
@@ -580,7 +580,7 @@ db_taur_NewToc(DBfile *_dbfile)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK void *
-db_taur_GetComponent(DBfile *_dbfile, char *obj_name, char *comp_name)
+db_taur_GetComponent(DBfile *_dbfile, char const *obj_name, char const *comp_name)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -650,7 +650,7 @@ db_taur_GetComponent(DBfile *_dbfile, char *obj_name, char *comp_name)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK DBmaterial *
-db_taur_GetMaterial(DBfile *_dbfile, char *mat_name)
+db_taur_GetMaterial(DBfile *_dbfile, char const *mat_name)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -789,7 +789,7 @@ db_taur_GetMaterial(DBfile *_dbfile, char *mat_name)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK DBucdmesh *
-db_taur_GetUcdmesh(DBfile *_dbfile, char *mesh_name)
+db_taur_GetUcdmesh(DBfile *_dbfile, char const *mesh_name)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1139,7 +1139,7 @@ db_taur_GetUcdmesh(DBfile *_dbfile, char *mesh_name)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK DBucdvar *
-db_taur_GetUcdvar(DBfile *_dbfile, char *var_name)
+db_taur_GetUcdvar(DBfile *_dbfile, char const *var_name)
 {
     int            i, j;
     int            nhex, nshell;
@@ -1182,7 +1182,7 @@ db_taur_GetUcdvar(DBfile *_dbfile, char *var_name)
 
     if (SILO_Globals.dataReadMask & DBUVData)
     {
-        uv->vals = (DB_DTPTR*) ALLOC_N(float *, 1);
+        uv->vals = (void **)ALLOC_N(void*, 1);
         if (taurus_readvar(taurus, var_name, &(((float **)(uv->vals))[0]), &uv->nels,
                            &uv->centering, meshname) < 0) {
             db_perror("taurus_readvar", E_CALLFAIL, me);
@@ -1197,7 +1197,7 @@ db_taur_GetUcdvar(DBfile *_dbfile, char *var_name)
          */
         if (taurus->activ >= 1000 && taurus->activ <= 1005) {
             if (uv->centering != 0) {
-                buf = uv->vals[0];
+                buf = (float *)uv->vals[0];
                 if (strcmp (meshname, "hs_mesh") == 0) {
                     for (i = 0, j = 0; i < taurus->nhex; i++) {
                         if (taurus->hex_activ [i] != 0) {
@@ -1274,7 +1274,7 @@ db_taur_GetUcdvar(DBfile *_dbfile, char *var_name)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK void *
-db_taur_GetVar(DBfile *_dbfile, char *varname)
+db_taur_GetVar(DBfile *_dbfile, char const *varname)
 {
     void          *ptr;
     char          *me = "db_taur_GetVar";
@@ -1315,7 +1315,7 @@ db_taur_GetVar(DBfile *_dbfile, char *varname)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK int
-db_taur_GetVarByteLength(DBfile *_dbfile, char *varname)
+db_taur_GetVarByteLength(DBfile *_dbfile, char const *varname)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1370,7 +1370,7 @@ db_taur_GetVarByteLength(DBfile *_dbfile, char *varname)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK int
-db_taur_GetVarLength(DBfile *_dbfile, char *varname)
+db_taur_GetVarLength(DBfile *_dbfile, char const *varname)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1424,7 +1424,7 @@ db_taur_GetVarLength(DBfile *_dbfile, char *varname)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK int
-db_taur_InqMeshname(DBfile *_dbfile, char *var_name, char *mesh_name)
+db_taur_InqMeshname(DBfile *_dbfile, char const *var_name, char *mesh_name)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1514,7 +1514,7 @@ db_taur_InqMeshname(DBfile *_dbfile, char *var_name, char *mesh_name)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK int
-db_taur_InqMeshtype(DBfile *_dbfile, char *mesh_name)
+db_taur_InqMeshtype(DBfile *_dbfile, char const *mesh_name)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1596,8 +1596,8 @@ db_taur_InqMeshtype(DBfile *_dbfile, char *mesh_name)
  *
  *-------------------------------------------------------------------------
  */
-SILO_CALLBACK int
-db_taur_InqVartype(DBfile *_dbfile, char *varname)
+SILO_CALLBACK DBObjectType 
+db_taur_InqVartype(DBfile *_dbfile, char const *varname)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1610,7 +1610,7 @@ db_taur_InqVartype(DBfile *_dbfile, char *varname)
         char *path = 0;
         int   vartype = DB_INVALID_OBJECT;
         int   changeDir = 0;
-        char *var = varname + pos;
+        char const *var = varname + pos;
         while(*var != '/' && var != varname)
             --var;
         /* If we have a slash in the name then we have a directory and 
@@ -1641,10 +1641,11 @@ db_taur_InqVartype(DBfile *_dbfile, char *varname)
         if(changeDir)
             db_taur_cd(taurus, pwd);
 
-        return vartype;
+        return (DBObjectType)vartype;
     }
     
-    return db_perror(varname, E_NOTFOUND, me);
+    db_perror(varname, E_NOTFOUND, me);
+    return DB_INVALID_OBJECT;
 }
 
 /*----------------------------------------------------------------------
@@ -1664,7 +1665,7 @@ db_taur_InqVartype(DBfile *_dbfile, char *varname)
  *--------------------------------------------------------------------*/
 /* ARGSUSED */
 SILO_CALLBACK int
-db_taur_InqVarExists(DBfile *_dbfile, char *varname)
+db_taur_InqVarExists(DBfile *_dbfile, char const *varname)
 {
     if (strcmp(varname, "time") == 0) {
         return (1);
@@ -1716,7 +1717,7 @@ db_taur_InqVarExists(DBfile *_dbfile, char *varname)
  *-------------------------------------------------------------------------
  */
 SILO_CALLBACK int
-db_taur_ReadVar(DBfile *_dbfile, char *varname, void *ptr)
+db_taur_ReadVar(DBfile *_dbfile, char const *varname, void *ptr)
 {
     DBfile_taur   *dbfile = (DBfile_taur *) _dbfile;
     TAURUSfile    *taurus = dbfile->taurus;
@@ -1769,7 +1770,7 @@ db_taur_ReadVar(DBfile *_dbfile, char *varname, void *ptr)
  *-------------------------------------------------------------------------
  */
 PRIVATE int
-db_taur_cd(TAURUSfile *taurus, char *path)
+db_taur_cd(TAURUSfile *taurus, char const *path)
 {
     int            i;
     int            state;
