@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -122,7 +122,7 @@ build_dbfile(DBfile *dbfile)
     int             meshtype=DB_QUADMESH;
     int             vartype=DB_QUADVAR;
     int             coord_type=DB_NONCOLLINEAR;
-    char           *coordnames[3];
+    char const * const coordnames[3] = {"xcoords", "ycoords", "zcoords"};
     int             ndims;
     int             dims[3], zdims[3];
     float          *coords[3];
@@ -228,9 +228,6 @@ build_dbfile(DBfile *dbfile)
      * Create the mesh.
      */
     meshname = "mesh1";
-    coordnames[0] = "xcoords";
-    coordnames[1] = "ycoords";
-    coordnames[2] = "zcoords";
     coords[0] = x;
     coords[1] = y;
     ndims = 2;
@@ -406,10 +403,10 @@ build_dbfile(DBfile *dbfile)
         DBAddOption(optlist, DBOPT_CYCLE, &cycle);
         DBAddOption(optlist, DBOPT_TIME, &time);
         DBAddOption(optlist, DBOPT_DTIME, &dtime);
-        DBAddOption(optlist, DBOPT_XLABEL, "X Axis");
-        DBAddOption(optlist, DBOPT_YLABEL, "Y Axis");
-        DBAddOption(optlist, DBOPT_XUNITS, "cm");
-        DBAddOption(optlist, DBOPT_YUNITS, "cm");
+        DBAddOption(optlist, DBOPT_XLABEL, (char *) "X Axis");
+        DBAddOption(optlist, DBOPT_YLABEL, (char *) "Y Axis");
+        DBAddOption(optlist, DBOPT_XUNITS, (char *) "cm");
+        DBAddOption(optlist, DBOPT_YUNITS, (char *) "cm");
 
         DBPutQuadmesh(dbfile, meshname, coordnames, coords, dims, ndims,
                       DB_FLOAT, DB_NONCOLLINEAR, optlist);
@@ -457,7 +454,7 @@ build_dbfile(DBfile *dbfile)
     DBAddOption(optlist, DBOPT_NMATNOS, &nmats);
     DBAddOption(optlist, DBOPT_MATNOS, matnos);
     /* For multi-species: */
-    DBAddOption(optlist, DBOPT_MATNAME, "mat1");
+    DBAddOption(optlist, DBOPT_MATNAME, (char *) "mat1");
     DBAddOption(optlist, DBOPT_NMAT, &nmats);
     DBAddOption(optlist, DBOPT_NMATSPEC, nmatspec);
     DBAddOption(optlist, DBOPT_SPECNAMES, species_names);
@@ -465,7 +462,7 @@ build_dbfile(DBfile *dbfile)
     
 
     /* create the multi-block mesh */
-    if (DBPutMultimesh(dbfile, "mesh1", nblocks, meshnames, 
+    if (DBPutMultimesh(dbfile, "mesh1", nblocks, (DBCAS_t) meshnames, 
                                         meshtypes, optlist) == -1)
     {
         DBFreeOptlist(optlist);
@@ -474,26 +471,26 @@ build_dbfile(DBfile *dbfile)
     }                           /* if */
  
    /* create the multi-block variables */
-    if (DBPutMultivar(dbfile, "d", nblocks, varnames[0], vartypes, optlist) == -1)
+    if (DBPutMultivar(dbfile, "d", nblocks, (DBCAS_t) varnames[0], vartypes, optlist) == -1)
     {
         DBFreeOptlist(optlist);
         fprintf(stderr, "Error creating multi var d\n");
         return (-1);
     }                           /* if */
-    if (DBPutMultivar(dbfile, "p", nblocks, varnames[1], vartypes, optlist) == -1)
+    if (DBPutMultivar(dbfile, "p", nblocks, (DBCAS_t) varnames[1], vartypes, optlist) == -1)
     {
         DBFreeOptlist(optlist);
         fprintf(stderr, "Error creating multi var p\n");
         return (-1);
     }                           /* if */
 
-    if (DBPutMultivar(dbfile, "u", nblocks, varnames[2], vartypes, optlist) == -1)
+    if (DBPutMultivar(dbfile, "u", nblocks, (DBCAS_t) varnames[2], vartypes, optlist) == -1)
     {
         DBFreeOptlist(optlist);
         fprintf(stderr, "Error creating multi var u\n");
         return (-1);
     }                           /* if */
-    if (DBPutMultivar(dbfile, "v", nblocks, varnames[3], vartypes, optlist) == -1)
+    if (DBPutMultivar(dbfile, "v", nblocks, (DBCAS_t) varnames[3], vartypes, optlist) == -1)
     {
         DBFreeOptlist(optlist);
         fprintf(stderr, "Error creating multi var v\n");
@@ -501,7 +498,7 @@ build_dbfile(DBfile *dbfile)
     }                           /* if */
 
     /* create the multi-block material */
-    if (DBPutMultimat(dbfile, "mat1", nblocks, matnames, optlist) == -1)
+    if (DBPutMultimat(dbfile, "mat1", nblocks, (DBCAS_t) matnames, optlist) == -1)
     {
         DBFreeOptlist(optlist);
         fprintf(stderr, "Error creating multi material\n");
@@ -509,7 +506,7 @@ build_dbfile(DBfile *dbfile)
     }                           /* if */
 
     /* create the multi-block species */
-    if (DBPutMultimatspecies(dbfile, "species1", nblocks, specnames, optlist) == -1)
+    if (DBPutMultimatspecies(dbfile, "species1", nblocks, (DBCAS_t) specnames, optlist) == -1)
     {
         DBFreeOptlist(optlist);
         fprintf(stderr, "Error creating multi species\n");
