@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -381,7 +381,7 @@ test_get_all(DBfile *dbfile)
     
     /* 1d char */
     puts("    1d_char");
-    if (NULL==(cdata_in=DBGetVar(dbfile, "1d_char"))) {
+    if (NULL==(cdata_in=(char *)DBGetVar(dbfile, "1d_char"))) {
 	puts("        DBGetVar() failed");
 	nerrors++;
     } else {
@@ -397,7 +397,7 @@ test_get_all(DBfile *dbfile)
 
     /* 2d short */
     puts("    2d_short");
-    if (NULL==(sdata_in=DBGetVar(dbfile, "2d_short"))) {
+    if (NULL==(sdata_in=(short*)DBGetVar(dbfile, "2d_short"))) {
 	puts("        DBGetVar() failed");
 	nerrors++;
     } else {
@@ -414,7 +414,7 @@ test_get_all(DBfile *dbfile)
 
     /* 3d int */
     puts("    3d_int");
-    if (NULL==(idata_in=DBGetVar(dbfile, "3d_int"))) {
+    if (NULL==(idata_in=(int*)DBGetVar(dbfile, "3d_int"))) {
 	puts("        DBGetVar() failed");
 	nerrors++;
     } else {
@@ -429,7 +429,7 @@ test_get_all(DBfile *dbfile)
 
     /* 3d long */
     puts("    3d_long");
-    if (NULL==(ldata_in=DBGetVar(dbfile, "3d_long"))) {
+    if (NULL==(ldata_in=(long*)DBGetVar(dbfile, "3d_long"))) {
 	puts("        DBGetVar() failed");
 	nerrors++;
     } else {
@@ -445,7 +445,7 @@ test_get_all(DBfile *dbfile)
     /* 4d float */
     DBForceSingle(TRUE);
     puts("    4d_float");
-    if (NULL==(fdata_in=DBGetVar(dbfile, "4d_float"))) {
+    if (NULL==(fdata_in=(float*)DBGetVar(dbfile, "4d_float"))) {
 	puts("        DBGetVar() failed");
 	nerrors++;
     } else {
@@ -462,7 +462,7 @@ test_get_all(DBfile *dbfile)
 
     /* 5d double */
     puts("    5d_double");
-    if (NULL==(ddata_in=DBGetVar(dbfile, "5d_double"))) {
+    if (NULL==(ddata_in=(double*)DBGetVar(dbfile, "5d_double"))) {
 	puts("        DBGetVar() failed");
 	nerrors++;
     } else {
@@ -876,7 +876,7 @@ test_dirs(DBfile *dbfile)
 	puts("DBWrite(d1c_A) failed");
 	nerrors++;
     }
-    if (DBGetDir(dbfile, curdir)<0 || strcmp(curdir, "/dir1/d1c")) {
+    if (DBGetDir(dbfile, curdir)<0 || strncmp(curdir, "/dir1/d1c", 9)) {
 	puts("DBGetDir() failed");
 	nerrors++;
     }
@@ -998,8 +998,8 @@ test_quadmesh(DBfile *dbfile)
 	nerrors++;
     }
 
-    if (DBPutQuadvar(dbfile, "qv1", "qm1", 3, varnames, (float**)vars, dims, 2,
-		     NULL, 0, DB_DOUBLE, DB_NODECENT, NULL)<0) {
+    if (DBPutQuadvar(dbfile, "qv1", "qm1", 3, (DBCAS_t) varnames,
+        vars, dims, 2, NULL, 0, DB_DOUBLE, DB_NODECENT, NULL)<0) {
 	puts("DBPutQuadmesh(qv1) failed");
 	nerrors++;
     }
@@ -1063,9 +1063,8 @@ test_ucdmesh(DBfile *dbfile)
     DBSetDir(dbfile, "/ucd");
 
     /* Mesh */
-    if (DBPutUcdmesh(dbfile, "um1", ndims, coordnames, (float**)coords, nnodes,
-		     nzones, "zl1", "fl1", DB_DOUBLE,
-		     NULL)<0) {
+    if (DBPutUcdmesh(dbfile, "um1", ndims, (DBCAS_t) coordnames,
+           coords, nnodes, nzones, "zl1", "fl1", DB_DOUBLE, NULL)<0) {
 	puts("DBPutUcdmesh(um1) failed");
 	nerrors++;
     }
@@ -1075,8 +1074,8 @@ test_ucdmesh(DBfile *dbfile)
     }
     
     /* Variable */
-    if (DBPutUcdvar(dbfile, "uv1", "um1", nvars, varnames, vars, nels, mixvars,
-		    mixlen, DB_FLOAT, DB_NODECENT, NULL)<0) {
+    if (DBPutUcdvar(dbfile, "uv1", "um1", nvars, (DBCAS_t) varnames,
+        vars, nels, mixvars, mixlen, DB_FLOAT, DB_NODECENT, NULL)<0) {
 	puts("DBPutUcdvar(uv1) failed");
 	nerrors++;
     }
