@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -75,6 +75,7 @@ be used for advertising or product endorsement purposes.
 #include <assert.h>
 #include <browser.h>
 #include <ctype.h>
+#include <stdint.h>
 #define MYCLASS(X)      ((obj_ary_t*)(X))
 
 typedef struct obj_ary_t {
@@ -112,7 +113,12 @@ typedef struct obj_ary_t {
                                  * sentinal, N, where N comes from
                                  * `(array "SH5 N, ...)' 
                                  */
-#define ARY_NSH         6       /*THIS MUST BE LAST!                    */
+#define ARY_SH_6        6       /* Array dimension determined by adding next
+                                 * argument to each of the specified dimension
+                                 * sizes as in `(array SH6 -1, self.dims)'
+                                 */
+                                    
+#define ARY_NSH         7       /*THIS MUST BE LAST!                    */
 
 static int      SHFlagsEncountered[ARY_NSH];
 
@@ -155,10 +161,10 @@ int             AryNProcessed;          /*num args used by ary_deref()  */
 class_t
 ary_class (void) {
 
-   class_t      cls = calloc (1, sizeof(*cls));
+   class_t      cls = (class_t)calloc (1, sizeof(*cls));
 
    cls->name = safe_strdup ("ARRAY");
-   cls->new = ary_new;
+   cls->newobj = ary_new;
    cls->copy = ary_copy;
    cls->dest = ary_dest;
    cls->apply = ary_apply;
@@ -198,7 +204,7 @@ ary_class (void) {
 static obj_t
 ary_new (va_list ap) {
 
-   obj_ary_t    *self = calloc (1, sizeof(obj_ary_t));
+   obj_ary_t    *self = (obj_ary_t*)calloc (1, sizeof(obj_ary_t));
    char         *s;
 
    assert (self);
@@ -243,7 +249,7 @@ ary_copy (obj_t _self, int flag) {
       retval = self;
 
    } else {
-      retval = calloc (1, sizeof(obj_ary_t));
+      retval = (obj_ary_t*)calloc (1, sizeof(obj_ary_t));
       retval->ascii_dims = safe_strdup (self->ascii_dims);
       retval->byte_offset = self->byte_offset;
       retval->ndims = self->ndims;
@@ -493,7 +499,7 @@ ary_walk1 (obj_t _self, void *mem, int operation, walk_t *wdata) {
    if ((nbytes=obj_sizeof(self->sub))<=0) {
       /* Total hack to make SH5 case work */
       if (ARY_SH_5==self->special_handling) {
-          nbytes=sizeof(int);
+          nbytes=sizeof(void*);
       } else {
           out_error ("unable to determine size of: ", self->sub);
           return;
@@ -591,27 +597,24 @@ ary_walk1 (obj_t _self, void *mem, int operation, walk_t *wdata) {
 
    case WALK_RETRIEVE:
       if (wdata->nvals<0) return ;      /*error already detected*/
-      if (wdata->vals == 0)
+      if (wdata->maxvals == -1)
       {
-          /* Total hack to make SH5 case work */
-          wdata->vals = (int *) malloc(10*sizeof(int));
+          wdata->vals = malloc(n*nbytes);
           wdata->nvals = 0;
-          wdata->maxvals = 10;
-          i = 0;
-          while (1)
-          {
-              if (wdata->nvals == wdata->maxvals)
-              {
-                  wdata->maxvals *= 1.5;
-                  wdata->vals = (int *) realloc(wdata->vals, wdata->maxvals*sizeof(int));
-              }
-              wdata->vals[i] = *((int*)((char*)mem+i*nbytes));
-              wdata->nvals++;
-              if (wdata->vals[i] == -1)
-                  break;
-              i++;
+          wdata->maxvals = n;
+          for (i=0; i<n; i++) {
+             obj_walk1 (self->sub, (char*)mem+i*nbytes, operation, wdata);
           }
       }
+      else if (wdata->vals == 0)
+      {
+          int sentinel = wdata->maxvals;
+          void **memtmp = (void**) mem;
+          i = 0;
+          while (sentinel != (intptr_t) memtmp[i])
+              i++;
+          wdata->nvals = i;
+      }
       else
       {
           for (i=0; i<n && wdata->nvals<wdata->maxvals; i++) {
@@ -1310,7 +1313,7 @@ ary_bind (obj_t _self, void *mem) {
 
    obj_ary_t    *self = MYCLASS(_self);
    char         *s, *t, *rest, buf[1024];
-   int          i, n, dim[NDIMS], max_dim0=0, sentinel;
+   int          i, n, dim[NDIMS], max_dim0=0, sentinel=0, dimadd=0;
    lex_t        *lex_input=NULL;
    obj_t        in=NIL, sdo=NIL;
    walk_t       wdata;
@@ -1368,9 +1371,14 @@ ary_bind (obj_t _self, void *mem) {
          } else if (ARY_SH_3==self->special_handling) {
             self->special_handling = 0; /*completely handled here*/
             max_dim0 = strtol (rest, &rest, 0);
+         } else if (ARY_SH_4==self->special_handling) {
+             ((void) 0); /* NO-OP */
          } else if (ARY_SH_5==self->special_handling) {
             sentinel = strtol (rest, &rest, 0);
-            self->ndims = 1;
+             self->ndims = 1;
+         } else if (ARY_SH_6==self->special_handling) {
+            self->special_handling = 0; /*completely handled here*/
+            dimadd = strtol (rest, &rest, 0);
          }
          assert (!rest || !*rest);
 
@@ -1409,20 +1417,40 @@ ary_bind (obj_t _self, void *mem) {
             self->ndims = 0;
             goto error;
          }
-         if (ARY_SH_5==self->special_handling)
+
+         if (ARY_SH_4==self->special_handling)
+         {
+             /* retriev underlying array and sum its values */
+             wdata.vals = 0;
+             wdata.nvals = 0;
+             wdata.maxvals = -1;
+             obj_walk1 (sdo, NULL, WALK_RETRIEVE, &wdata);
+             self->ndims = 1;
+             self->dim[0] = 0;
+             if (wdata.vals && wdata.nvals > 0)
+             {
+                 for (i=0; i<wdata.nvals; i++)
+                     self->dim[0] += wdata.vals[i];
+                 free(wdata.vals);
+             }
+             sdo = obj_dest (sdo);
+         }
+         else if (ARY_SH_5==self->special_handling)
          {
              wdata.vals = 0;
              wdata.nvals = 0;
-             wdata.maxvals = 0;
+             wdata.maxvals = sentinel;
              obj_walk1 (sdo, NULL, WALK_RETRIEVE, &wdata);
-             if (wdata.nvals<1) {
-                out_errorn ("array dimension has no value: %s", t);
+             if (wdata.nvals<1)
+             {
                 self->ndims = 0;
-                goto error;
+             }
+             else
+             {
+                 self->ndims = 1;
+                 self->dim[0] = wdata.nvals;
              }
              sdo = obj_dest (sdo);
-             self->ndims = 1;
-             self->dim[0] = wdata.nvals;
          }
          else
          {
@@ -1430,7 +1458,7 @@ ary_bind (obj_t _self, void *mem) {
              wdata.nvals = 0;
              wdata.maxvals = NELMTS(dim);
              obj_walk1 (sdo, NULL, WALK_RETRIEVE, &wdata);
-             if (wdata.nvals<1) {
+             if (wdata.nvals<0) {
                 out_errorn ("array dimension has no value: %s", t);
                 self->ndims = 0;
                 goto error;
@@ -1441,11 +1469,11 @@ ary_bind (obj_t _self, void *mem) {
                    out_errorn ("too many dimensions in array type");
                    goto error;
                 }
-                self->dim[self->ndims] = n = wdata.vals[i];
+                self->dim[self->ndims] = n = wdata.vals[i] + dimadd;
                 self->ndims += 1;
-                if (n<0) {
-                   out_errorn ("dimension %d is invalid: %d",
-                               self->ndims, n);
+                if (n<0 && !(wdata.vals[i]==0 && dimadd==-1)) {
+                   out_errorn ("dimension %d is invalid: %d (%s)",
+                               self->ndims, n, t);
                    goto error;
                 }
              }
