@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -1223,7 +1223,7 @@ out_reset (out_t *f) {
 out_t *
 out_stream (FILE *stream) {
 
-   out_t        *f = calloc (1, sizeof(out_t));
+   out_t        *f = (out_t *)calloc (1, sizeof(out_t));
 
    assert (f);
    if (NULL==(f->f=fdopen(fileno(stream), "w"))) {
