@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -63,6 +63,7 @@ be used for advertising or product endorsement purposes.
  *
  *-------------------------------------------------------------------------
  */
+#include <config.h>
 #include <browser.h>
 #include <ctype.h>
 #include <signal.h>
@@ -172,9 +173,9 @@ parse_init (void) {
         "use of this function is with the `ls' or `diff' functions which can "
         "take file objects as arguments. For instance, the command `ls "
         "(file curv2d.pdb)' opens the file, lists the root directory, and "
-        "then closes the file. The file is opened for read-write unless "
-        "the file has only read permission or the `$rdonly' variable is "
-        "non-zero and non-nil.  See also `help open'.");
+        "then closes the file. The file is opened for read-only unless "
+        "the `$writable' variable is non-zero and non-nil and the file has "
+        "write permission. See also `help open'.");
         
    bif ("help",         V_help,         HOLDFIRST,
         "Show documentation.",
@@ -198,8 +199,9 @@ parse_init (void) {
    
    bif ("ls",           V_list,         HOLD,
         "List directory contents.",
-        "Lists the current working directory (CWD) of the specified file. If "
-        "no file is specified then list the CWD of the file represented by "
+        "Lists the current working directory (CWD) of the specified file. "
+        "Pass a -l option to expand any symbolic links to show their targets. "
+        "If no file is specified then list the CWD of the file represented by "
         "browser internal variable `$1'. Subsequent arguments are the names "
         "of the objects to be listed. If no names are supplied then all "
         "objects of the CWD are listed. Shell wild-cards are allowed in the "
