@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1994 - 2010, Lawrence Livermore National Security, LLC.
+Copyright (C) 1994-2016 Lawrence Livermore National Security, LLC.
 LLNL-CODE-425250.
 All rights reserved.
 
@@ -136,6 +136,7 @@ prim_assoc_t    PA_BR_OBJTYPE[] = {
    {BROWSER_DB_MRGTREE,         "mrgtree"},
    {BROWSER_DB_GROUPELMAP,      "groupelmap"},
    {BROWSER_DB_MRGVAR,          "mrgvar"},
+   {BROWSER_DB_SYMLINK,         "symlink"},
    {0,                          NULL}};
 
 prim_assoc_t    PA_OBJTYPE[] = {
@@ -316,6 +317,24 @@ prim_assoc_t PA_REPRBLOCK[] = {
    {-1,        "not specified"},
    {0,         NULL}};
 
+prim_assoc_t PA_MISSING_VALUE[] = {
+   {PA_NAME,                                    "missing value"},
+   {DB_MISSING_VALUE_NOT_SET, "not set"},
+   {0,                        NULL}};
+
+prim_assoc_t PA_ZONETYPE[] = {
+   {PA_NAME,                                    "zonetype"},
+   {DB_ZONETYPE_BEAM,        "BEAM"},
+   {DB_ZONETYPE_POLYGON,     "POLYGON"},
+   {DB_ZONETYPE_TRIANGLE,    "TRIANGLE"},
+   {DB_ZONETYPE_QUAD,        "QUAD"},
+   {DB_ZONETYPE_POLYHEDRON,  "POLYHEDRON"},
+   {DB_ZONETYPE_TET,         "TET"},
+   {DB_ZONETYPE_PYRAMID,     "PYRAMID"},
+   {DB_ZONETYPE_PRISM,       "PRISM"},
+   {DB_ZONETYPE_HEX,         "HEX"},
+   {0,                        NULL}};
+
 
 /*-------------------------------------------------------------------------
  * Function:    prim_class
@@ -340,10 +359,10 @@ prim_assoc_t PA_REPRBLOCK[] = {
 class_t
 prim_class (void) {
 
-   class_t      cls = calloc (1, sizeof(*cls));
+   class_t      cls = (class_t)calloc (1, sizeof(*cls));
 
    cls->name = safe_strdup ("PRIM");
-   cls->new = prim_new;
+   cls->newobj = prim_new;
    cls->copy = prim_copy;
    cls->dest = prim_dest;
    cls->objname = prim_name;
@@ -383,7 +402,7 @@ prim_class (void) {
 static obj_t
 prim_new (va_list ap) {
 
-   obj_prim_t   *self = calloc (1, sizeof(obj_prim_t));
+   obj_prim_t   *self = (obj_prim_t *)calloc (1, sizeof(obj_prim_t));
    char         *s;
 
    assert (self);
@@ -423,7 +442,7 @@ prim_copy (obj_t _self, int flag) {
       retval = self;
 
    } else {
-      retval = calloc (1, sizeof(obj_prim_t));
+      retval = (obj_prim_t *)calloc (1, sizeof(obj_prim_t));
       retval->name = safe_strdup (self->name);
       retval->tname = self->tname ? safe_strdup (self->tname) : NULL;
       retval->browser_type = self->browser_type;
@@ -842,7 +861,12 @@ prim_walk1 (obj_t _self, void *mem, int operation, walk_t *wdata)
                 break;
                 
             case BROWSER_DOUBLE:
-                if (16==obase) {
+                for (i=0; self->assoc && self->assoc[i].s; i++) {
+                    if (*((double*)mem) == self->assoc[i].n) break;
+                }
+                if (self->assoc && self->assoc[i].s) {
+                    out_printf(f, "%s", self->assoc[i].s);
+                } else if (16==obase) {
                     for (i=0; i<sizeof(double); i++) {
                         sprintf(buf+2*i, "%02x", *((unsigned char*)mem+i));
                     }
@@ -922,6 +946,7 @@ prim_getval_ll(int type, void *mem)
     case BROWSER_FLOAT:     return (long long) *((float*)mem);
     case BROWSER_DOUBLE:    return (long long) *((double*)mem);
     }
+    return 0;
 }
 
 /*-------------------------------------------------------------------------
@@ -947,6 +972,7 @@ prim_getval(int type, void *mem)
     case BROWSER_FLOAT:     return (double) *((float*)mem);
     case BROWSER_DOUBLE:    return (double) *((double*)mem);
     }
+    return 0;
 }
 
 
@@ -1436,6 +1462,12 @@ prim_bind (obj_t _self, void *mem) {
          self->nbytes = sizeof(double);
          break;
 
+      case DB_NOTYPE:
+         self->tname = safe_strdup ("notype");
+         self->browser_type = BROWSER_INT;
+         self->nbytes = 0;
+         break;
+
       default:
          out_errorn ("prim_bind: cannot resolve %s to a silo type",
                      self->name);
